<?php

    class Send_Email_Notification_Action extends Reactor_Action
    {
        protected $tpl = 'send_email_notification.php';
        protected $name = 'Send an email notification';
		
		public static $registration = array(
			ACTION_NAME => 'Send an email notification',
			ACTION_CATEGORY => 'Network',
			ACTION_DESCRIPTION => 'Sends an email message to a list of recipients.',
			ACTION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '01/22/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array(),	//Misc information	
		); 		
		
        protected $fields = array(
            'recips' => array(
                'label' => 'recipients',
                'validate' => 'trim|required|callback_check_emails'
            ),
            'subject' => array(
                'label' => 'subject',
                'validate' => 'trim|required'
            ),
            'message' => array(
                'label' => 'mail message',
                'validate' => 'trim|required'
            ),
            'local_file' => array(
                'label' => 'local file',
                'validate' => 'trim|callback_check_filename',
            ),

        );

       public function get_label()
        {
			
			if($params = $this->get_params()) {
			
				$recips = $params['recips'];
				$subject = $params['subject'];
								
				return sprintf('Send an email notification to \'%s\' with subject \'%s\'',$recips,$subject);
				}

            return parent::get_label();
        }


        public function check_emails($str)
        {
            foreach(explode(",", $str) as $email) {
                $e = explode("@", trim($email));
                if(count($e) != 2  || trim($e[0]) == false || trim($e[1]) == false) {
                    $this->ci->form_validation->set_message('check_emails', 'One or more recipient addresses was invalid.');      
                    return FALSE;
                }
            }
            return TRUE;
        }

        /**
         * Sanity check files
         **/
        public function check_filename($str)
        {
            if(!strlen($str)) {
                return TRUE;
            }
            $banned_files = array(
                '/etc/passwd',
                '/etc/hosts',
                '/etc/resolv.conf',
                '/etc/network/interfaces',
                '/etc/sysconfig/network',
            );

            // check for path traversal (dot slash)
            $exp = '/(\.\/)/';
            if(preg_match($exp, $str)) {
                $this->ci->form_validation->set_message('check_filename', 'The filename must be absolute.');
                return FALSE;
            }

            // check for a generally acceptable file
            $exp = '/^\/([^\0]+\/)+([^\0]+)$/';
            if(!preg_match($exp, $str)) {
                $this->ci->form_validation->set_message('check_filename', 'The filename was invalid. Please ensure you have entered a valid, absolute UNIX filename and path.');
                return FALSE;
            }
                 
            // check against banned list
            foreach($banned_files as $file) {
                if($file == $str) {
                    $this->ci->form_validation->set_message('check_filename', 'Uploading the file specified is not allowed.');
                    return FALSE;
                }
            }
            return TRUE;
        }

        public function run($context)
        {
            $this->set_context($context);
            $message = $this->get_param('message');
            $subject = $this->get_param('subject');
            $recips = explode(",", $this->get_param('recips'));
            $this->log('info', "sending email notification to ". $this->get_param('recips'));
            if($file = $this->get_param('local_file')) {
                $this->log('info', 'Attaching file '. $file);
                if(!file_exists($file)) {
                    $this->log('error', sprintf("The file %s does not exist.", $file));
                    $this->set_result(1);   
                    return $context;
                }
            } else {
                $file = null;
            }
							
			if(ENVIRONMENT=='development')
				logit("MESSAGE: ".$message);				
			
            if(send_email_notification($recips, $subject, $message, null, null, array($file))) {
                $status = 0;
            } else {
                $status = 1;
            }
            $this->set_result($status);
            return $context;
        }
    }
